#include <stdio.h>
#include <ctype.h>
#define TRUE      1
#define FALSE     0
#define MAXENG    3
#define MAXT      64

FILE *efile, *fopen();

char efile_name[] = {"rasp.eng"}; /* name of engine database file */

struct engine {
    double t2;         /* thrust duration    (seconds)                 */
    double m2;         /* propellant wt.     (kilograms)               */
    double wt;         /* initial engine wt. (kilograms)               */
    double thrust[MAXT]; /* thrust curve     (Time-sec,Thrust-Newtons) */
    double navg;       /* average thrust     (newtons)                 */
    double ntot;       /* total impulse      (newton-seconds)          */
    double npeak;      /* peak thrust        (newtons)                 */
    int   dia;         /* engine diameter    (millimeters)             */
    int   len;         /* engine length      (millimeters)             */
    int   delay[8];    /* ejection delays available (sec)              */
    char  mfg[32];     /* manufacturer info                            */
    char  code[32]     /* engine name                                  */
       };

struct engine e_info[MAXENG];

load_engine(e) int e; {
  int t, dia, len, more;
  char s[256], *c, sdelay[32];
  double sum,peak, t1, t2, f1, f2;
  double *p;

  for(more = TRUE; more;) {
    if((c = fgets(s, 256, efile)) != NULL) {
      if(*c != ';') {
        if(sscanf(s, "%s %d %d %s %lf %lf %s", e_info[e].code, &e_info[e].dia,
                  &e_info[e].len, sdelay, &e_info[e].m2, &e_info[e].wt,
                  e_info[e].mfg) != 7) {
          printf("\n*** Error - Bad line in rasp.eng\n");
          exit();
        }
        more = FALSE;
      }
    }
    else
      return(-1);
  }
  t = 0;
  for(more = TRUE; more;) {
    if((c = fgets(s, 256, efile)) != NULL) {
      if(*c != ';') {
        if(t >= MAXT) {
          printf("\n*** Error - Thrust table overflow in rasp.eng\n");
          exit();
        }
        if(sscanf(s, "%lf %lf",
                  &e_info[e].thrust[t], &e_info[e].thrust[t+1]) != 2) {
          printf("\n*** Error - Bad thrust table line in rasp.eng\n");
          exit();
        }
        if(e_info[e].thrust[t+1] == 0) {
          e_info[e].t2 = e_info[e].thrust[t];
          more = FALSE;
        }
        else
          t += 2;
      }
    }
    else {
      printf("\n*** Error - unexpected EOF while loading thrust curve.\n");
      exit();
    }
  }
  t1 = f1 = sum = peak = 0.0;
  p = &e_info[e].thrust[0];
  do {
    t2 = *p++;
    f2 = *p++;
    if(f2 > peak) peak = f2;
    sum += (t2 - t1) * (f1 + f2) * 0.5;
    t1 = t2;
    f1 = f2;
  } while (f1 != 0.0);
  e_info[e].ntot = sum;
  e_info[e].npeak = peak;
  e_info[e].navg = sum / t1;
  c = sdelay;
  for(t = 0; t < 7; t++) {
    e_info[e].delay[t] = (int) strtol(c, &c, 10);
    if(*c != '-') {
      e_info[e].delay[t+1] = -1;
      break;
    }
    else
      c++;

  }
  return(e);
}

print_engine_info(e)
     int e;
{
  int d;
  printf("%-6s %-6s\t%dx%d\t%.2lf\t%.2lf\t%.2lf\t%.2lf\t",
         e_info[e].code, e_info[e].mfg,
         e_info[e].dia, e_info[e].len,
         e_info[e].navg, e_info[e].ntot,
         e_info[e].npeak, e_info[e].t2);
  for(d = 0; e_info[e].delay[d] >= 0; d++)
    printf("%d ", e_info[e].delay[d]);
  printf("\n");
}

print_engine_header() {
  printf("\n\t\tSize\tAverage\tTotal\tPeak\tBurn\n");
  printf("\t\tDia x Len\tThrust\tImpulse\tThrust\tTime\tDelays\n");
  printf("Type  Mfg/Opt\t mm \t (Nt)\t(Nt-Sec) (Nt)\t(Sec)\t(Sec)\n");
  printf("----  -------\t------\t------\t-------\t------\t-----\t------\n");
}

main() {

  if((efile = fopen(efile_name,"r")) == NULL) {
    printf("\n*** Error - unable to open engine database %s ***\n",
           efile_name);
    exit();
  }

  print_engine_header();
  while(load_engine(0) != -1)
    print_engine_info(0);
  exit(0);
}
